@@ -12,7 +12,8 @@ use warnings;
 
 use base 'Class::Accessor::Fast';
 
-our $VERSION = '0.40';
+our $VERSION = '0.40_02';
+$VERSION = eval $VERSION;
 
 =head1 DESCRIPTION
 
@@ -70,12 +71,25 @@ together multiple egdes into one path.
 
 =item origin
 
+  $edge->origin( $name );
+
+  $name = $edge->origin;
+
 An accessor method for identifying the I<origin vertex>.
 
 =item depart_time
 
+  $edge->depart_time( $time );
+
+  $time = $edge->depart_time;
+
 An accessor method for specifying the I<depature time>, as a number.
 
+The number value can be anything you need it to be: reals or integers,
+Unix epoch time, the number of days since X day, shire years, seconds,
+minutes, hours, days, weeks, whatever.  As long as all edges have time
+values in the same units.
+
 =item destination
 
 An accessor method for identifying the I<destination vertex>.
@@ -83,7 +97,8 @@ An accessor method for identifying the I<destination vertex>.
 =item arrive_time
 
 An accessor method for specifying the I<arrival time>, as a number. It
-is assumed to be greater than or equal to the depature time.
+is assumed to be greater than or equal to the depature time. (See
+L</depart_time> for more information on the format.)
 
 =item data
 
@@ -119,9 +134,10 @@ sub copy {
   my $self = shift;
   my $copy = __PACKAGE__->new();
   foreach my $method (qw(
-    path_id origin depart_time destination arrive_time data )) {
-    $copy->$method( $self->$method );
-  }
+      id path_id origin depart_time destination arrive_time data ))
+    {
+      $copy->$method( $self->$method ), if (defined $self->$method);
+    }
   return $copy;
 }
 
